/*
 * Moblin-Web-Browser: The web browser for Moblin
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 */

Components.utils.import("resource://gre/modules/utils.js");

const nsISupports = Components.interfaces.nsISupports;
const nsISupportsWeakReference = Components.interfaces.nsISupportsWeakReference;

var searchStr = "";

function binaryToHex(aRaw)
{
  return [('0' + aRaw.charCodeAt(i).toString(16)).slice(-2)
          for (i in aRaw)].join('');
}

// Caculate MD5 checksum for a string
function caculateMD5CheckSum(aString)
{
  if (!aString) {
      aString = "";
  }

  var hasher = Components.classes["@mozilla.org/security/hash;1"]
                         .createInstance(Components.interfaces.nsICryptoHash);
  hasher.init(Components.interfaces.nsICryptoHash.MD5);

  var stream = Components.classes['@mozilla.org/io/string-input-stream;1']
                         .createInstance(Components.interfaces.nsIStringInputStream);
  stream.setData(aString, aString.length);
  hasher.updateFromStream(stream, stream.available());

  return binaryToHex(hasher.finish(false));
}

function getThumbPath (uri)
{
  var prefs = Components.classes["@mozilla.org/preferences-service;1"]
    .getService(Components.interfaces.nsIPrefBranch);

  if (prefs.getBoolPref("moblin.thumbnails.enabled")) {
    var dirService = Components.classes["@mozilla.org/file/directory_service;1"].
      getService(Components.interfaces.nsIProperties);
    var homeDir = dirService.get("Home", Components.interfaces.nsIFile).path;
    
    var thumbSubDir = ".thumbnails/large";
    thumbSubDir = prefs.getComplexValue("moblin.thumbnails.path",
					       Components.interfaces.nsISupportsString).data;

    return homeDir + "/" + thumbSubDir + "/" + caculateMD5CheckSum(uri) + ".png";
  }
}

function searchInString(haystack, needle)
{
  var i;

  if (needle.length <= haystack.length)
    for (i = 0; i <= haystack.length - needle.length; i++)
      {
        var j;
        for (j = 0; j < needle.length; j++)
          if (haystack.charAt(i + j) != needle.charAt(j))
            break;
        if (j >= needle.length)
          return i;
      }

  return -1;
}

function deleteBookmark (url)
{
  var transService = Components.classes["@mozilla.org/browser/placesTransactionsService;1"]
    .getService(Components.interfaces.nsIPlacesTransactionsService);
  var ioService = Components.classes["@mozilla.org/network/io-service;1"]  
    .getService(Components.interfaces.nsIIOService);
  var uri = ioService.newURI(url, null, null);
  var itemIds = PlacesUtils.getBookmarksForURI(uri);
  for (var i=0; i < itemIds.length; i++) {
    transService.doTransaction(transService.removeItem(itemIds[i]));
  }
  rebuildPinnedPagesList ();
}

function rebuildPinnedPagesList()
{
  var default_msg_div = document.getElementById('pinned_pages_default_msg');
  var pinnedPagesDiv = document.getElementById("pinned_pages_div");

  // Remove all existing pinned pages
  pinnedPagesDiv.innerHTML = "";

  var haveSearchStr = searchStr && searchStr.match(/\S/);

  var historyService = Components.classes["@mozilla.org/browser/nav-history-service;1"]
    .getService(Components.interfaces.nsINavHistoryService);
  var db = historyService.QueryInterface(Components.interfaces.nsPIPlacesDatabase).DBConnection;
  var q = "SELECT DISTINCT b.title as title, p.url as uri " + 
          "FROM moz_bookmarks as b INNER JOIN moz_places as p ON b.fk = p.id " + 
                                  "LEFT JOIN moz_favicons as f ON p.favicon_id = f.id " + 
          "WHERE b.type = 1 AND p.hidden <> 1 AND " +
                "b.parent <> (SELECT ifnull((SELECT id FROM moz_bookmarks WHERE title = 'Mozilla Firefox' LIMIT 1), -1)) AND " +
                "b.title <> 'Get Bookmark Add-ons' AND " + 
                "b.title <> 'Getting Started' AND " +
                "SUBSTR(p.url, 1, 7) <> 'moblin:' " +
          "ORDER BY p.visit_count DESC";

  var stmt;
  stmt = db.createStatement(q);
  while (stmt.executeStep()) {
    var page = {uri: "", title: ""};
    page.title = stmt.getString(0);
    page.uri = stmt.getString(1);

    var lowerSearchStr;
    var titleMatch;

    if (haveSearchStr)
      lowerSearchStr = searchStr.trim().toLowerCase();
    
    if (!lowerSearchStr ||
        (titleMatch = searchInString(page.title.toLowerCase(),
                                     lowerSearchStr)) != -1)
      {
        var div = document.createElement("div");
        div.setAttribute("class", "pinned_page_div");

        var a = document.createElement("a");
        //a.setAttribute("href", page.uri);
      
        var thumbnailDiv = document.createElement("div");
        thumbnailDiv.setAttribute("class", "pinned_page_thumbnail thumbnail");
      
        // Create elements for the close button. The outer div is
        // just used to make the position for the inner element
        // relative to the top of the thumbnail
        var closeOuter = document.createElement('div');
        closeOuter.setAttribute('class', 'close-button-outer');
        var closeInner = document.createElement('a');
        closeInner.setAttribute('class', 'close-button-inner');
        closeInner.setAttribute('href', 'javascript:');
        closeOuter.appendChild(closeInner);
        thumbnailDiv.appendChild(closeOuter);
      
        // Create a closure with the url and use it as a callback
        // to unpin the page when the link is clicked
        let (url = page.uri) {
          closeInner.addEventListener("click",
                                      function(e) {
                                        deleteBookmark (url);
                                        e.stopPropagation();
                                      },
                                      false);
        }
      
        var img = document.createElement("img");
        img.setAttribute("class", "thumbnail");
        img.setAttribute("src", "file://" + getThumbPath (page.uri));
        thumbnailDiv.appendChild(img);
      
        a.appendChild(thumbnailDiv);
      
        img = document.createElement("img");
        img.setAttribute("class", "pinned_page_favicon");
        if (page.icon)
          img.setAttribute("src", page.icon.spec);
        a.appendChild(img);
      
        var span = document.createElement("span");
        span.setAttribute("class", "pinned_page_title");
        if (titleMatch != undefined)
          {
            var title = page.title;
            var preMatch =
              document.createTextNode(title.slice(0, titleMatch));
            span.appendChild(preMatch);
            var match =
              document.createTextNode(title.slice(titleMatch,
                                                  lowerSearchStr.length +
                                                  titleMatch));
            var matchSpan = document.createElement("span");
            matchSpan.setAttribute("class", "search-match");
            matchSpan.appendChild(match);
            span.appendChild(matchSpan);
            var postMatch =
              document.createTextNode(title.slice(titleMatch +
                                                  lowerSearchStr.length));
            span.appendChild(postMatch);
          }
        else
          span.appendChild(document.createTextNode(page.title));

        a.appendChild(span);

        a.appendChild(document.createElement("br"));
      
        div.appendChild(a);
	
        pinnedPagesDiv.appendChild(div);
      }
  }
  
  if (pinnedPagesDiv.childNodes.length != 0)
    default_msg_div.setAttribute('style', 'display: none');
  else
    default_msg_div.setAttribute('style', 'display: block');

  stmt.reset();
  stmt.finalize();
  stmt = null;

}

function updateSearch()
{
  // Restart the search
  searchStr = document.getElementById("search-entry").value;
  rebuildPinnedPagesList();
}

function searchEntryKeyDownCb(event)
{
  // If there is a search queued then run it immediately when enter is
  // pressed
  if (event.keyCode == 13)
    updateSearch();
}

function updateSearchClearButton()
{
  var searchArea = document.getElementById("search-area");
  var searchEntry = document.getElementById("search-entry");

  var oldClass = searchArea.className;

  // Remove any previous mention of the 'emtpy' or 'non-empty' class
  var match = /\s*\b(non-)?empty\b/.exec(oldClass);
  if (match)
    oldClass = (oldClass.slice(0, match.index) +
                oldClass.slice(match.index + match[0].length));

  // Hide the clear button if there is only whitespace in the search box
  searchArea.className = (oldClass + " " +
                          (searchEntry.value.match(/\S/) ?
                           "non-empty" : "empty"));
}

function textChangedCb()
{
  updateSearchClearButton();
  updateSearch ();
}

function clearSearchButtonClickCb()
{
  var searchEntry = document.getElementById("search-entry");
  searchEntry.value = "";
  updateSearchClearButton();
  updateSearch();
}

function bookmarksObserver ()
{
  dump ("bookmarksObserver()\n");
}

bookmarksObserver.prototype = 
{
  onBeginUpdateBatch : function ()
  {
  },

  onEndUpdateBatch : function ()
  {
  },

  onItemAdded : function (aItemId, aParentID, aIndex, aItemType)
  {
    rebuildPinnedPagesList ();
  },

  onItemRemoved : function (aItemId, aParentID, aIndex, aItemType)
  {
    rebuildPinnedPagesList ();
  },

  onItemChanged : function (aBookmarkId, aProperty, aIsAnnotationProperty, aNewValue, aItemType)
  {
    rebuildPinnedPagesList ();
  },

  onItemMoved : function (aItemId, aOldParentId, aOldIndex, aNewParentId, aNewIndex, aItemType)
  {
    rebuildPinnedPagesList ();
  },

  onItemVisited : function (aBookmarkId, aVisitID, time)
  {
  },

  QueryInterface: function(iid) {
    if (iid.equals(Components.interfaces.nsINavBookmarkObserver) ||
        iid.equals(Components.interfaces.nsISupports)) {
      return this;
    }
    throw Components.results.NS_ERROR_NO_INTERFACE;
  }
}

function eventListener (event )
{
  updateSearch ();

  var searchEntry = document.getElementById("search-entry");
  searchEntry.addEventListener("keydown", searchEntryKeyDownCb, false);
  searchEntry.addEventListener("input", textChangedCb, false);
  updateSearchClearButton();
  var clearSearchButton = document.getElementById("clear-search-button");
  clearSearchButton.addEventListener("click", clearSearchButtonClickCb, false);
  
  var bookmarkService = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"]
    .getService(Components.interfaces.nsINavBookmarksService);
  var bmObserver = new bookmarksObserver ();
  bookmarkService.addObserver (bmObserver, false);
  
  // Unregister the load event so we don't initialize more than once
  event.currentTarget.removeEventListener("DOMContentLoaded",
					  arguments.callee,
					  false);
}

// Run the initialisation once the page has finished loading
document.addEventListener("DOMContentLoaded", eventListener, false);
