/*
 * Moblin-Web-Browser: The web browser for Moblin
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 */

const nsISupports = Components.interfaces.nsISupports;
const nsISupportsWeakReference = Components.interfaces.nsISupportsWeakReference;
const N_PINNED_PAGES = 9;
const N_FAV_PAGES = 9;

function binaryToHex(aRaw)
{
  return [('0' + aRaw.charCodeAt(i).toString(16)).slice(-2)
          for (i in aRaw)].join('');
}

// Caculate MD5 checksum for a string
function caculateMD5CheckSum(aString)
{
  if (!aString) {
      aString = "";
  }

  var hasher = Components.classes["@mozilla.org/security/hash;1"]
                         .createInstance(Components.interfaces.nsICryptoHash);
  hasher.init(Components.interfaces.nsICryptoHash.MD5);

  var stream = Components.classes['@mozilla.org/io/string-input-stream;1']
                         .createInstance(Components.interfaces.nsIStringInputStream);
  stream.setData(aString, aString.length);
  hasher.updateFromStream(stream, stream.available());

  return binaryToHex(hasher.finish(false));
}

function getThumbPath (uri)
{
  var prefs = Components.classes["@mozilla.org/preferences-service;1"]
    .getService(Components.interfaces.nsIPrefBranch);

  if (prefs.getBoolPref("moblin.thumbnails.enabled")) {
    var dirService = Components.classes["@mozilla.org/file/directory_service;1"].
      getService(Components.interfaces.nsIProperties);
    var homeDir = dirService.get("Home", Components.interfaces.nsIFile).path;
    
    var thumbSubDir = ".thumbnails/large";
    thumbSubDir = prefs.getComplexValue("moblin.thumbnails.path",
					       Components.interfaces.nsISupportsString).data;

    return homeDir + "/" + thumbSubDir + "/" + caculateMD5CheckSum(uri) + ".png";
  }
}

function buildPinnedPages()
{
  var iconsvc = Components.classes["@mozilla.org/browser/favicon-service;1"].
                  getService(Components.interfaces.nsIFaviconService);

  var div = document.getElementById('pinned_pages_div');
  var default_msg_div = document.getElementById('pinned_pages_default_msg');
  var manage_pinned_pages_msg_div = document.getElementById('pinned_pages_manage_msg');

  // Clear all existing entries
  while (div.childNodes.length > 0)
    div.removeChild(div.childNodes[0]);

  var historyService = Components.classes["@mozilla.org/browser/nav-history-service;1"]
    .getService(Components.interfaces.nsINavHistoryService);
  var db = historyService.QueryInterface(Components.interfaces.nsPIPlacesDatabase).DBConnection;
  var q = "SELECT DISTINCT b.title as title, p.url as uri, f.url as favicon " + 
          "FROM moz_bookmarks as b INNER JOIN moz_places as p ON b.fk = p.id " + 
                                  "LEFT JOIN moz_favicons as f ON p.favicon_id = f.id " + 
          "WHERE b.type = 1 AND p.hidden <> 1 AND " +
                "b.parent <> (SELECT ifnull((SELECT id FROM moz_bookmarks WHERE title = 'Mozilla Firefox' LIMIT 1), -1)) AND " +
                "b.title <> 'Get Bookmark Add-ons' AND " + 
                "b.title <> 'Getting Started' AND " +
                "SUBSTR(p.url, 1, 7) <> 'moblin:' " +
          "ORDER BY p.visit_count DESC LIMIT " + (N_PINNED_PAGES);

  var stmt;
  stmt = db.createStatement(q);
  while (div.childNodes.length < N_PINNED_PAGES && stmt.executeStep()) {
    var page = {uri: "", title: "", favicon: ""};
    page.title = stmt.getString(0);
    page.uri = stmt.getString(1);
    page.favicon = stmt.getString(2);

    var a = document.createElement('a');
    var pageDiv = document.createElement('div');
    var imgDiv = document.createElement('div');
    var img = document.createElement('img');
    var innerDiv = document.createElement('div');
    var titleSpan = document.createElement('span');

    a.setAttribute('href', page.uri);
    pageDiv.setAttribute('class', 'pinned_page_div');
    imgDiv.setAttribute('class', 'pinned_favicon_div');

    if (page.favicon) {
      var uri = Components.classes["@mozilla.org/network/simple-uri;1"]
        .createInstance(Components.interfaces.nsIURI);
      uri.spec = page.favicon;

      faviconURI = iconsvc.getFaviconDataAsDataURL(uri);
      if (faviconURI){
        img.setAttribute('class', 'pinned_favicon');
        img.setAttribute('src', faviconURI);
      }
    }
    
    imgDiv.appendChild(img);
    pageDiv.appendChild(imgDiv);
    innerDiv.setAttribute('class', 'pinned_inner_div');
    titleSpan.setAttribute('class', 'pinned_title');
    titleSpan.appendChild(document.createTextNode(page.title + " "));
    innerDiv.appendChild(titleSpan);
    pageDiv.appendChild(innerDiv);
    a.appendChild(pageDiv);
    div.appendChild(a);
  }

  if(div.childNodes.length == 0){
    default_msg_div.setAttribute('style', 'display: block');
    manage_pinned_pages_msg_div.setAttribute('style', 'display: none');
  }
  else{
    default_msg_div.setAttribute('style', 'display: none');
    manage_pinned_pages_msg_div.setAttribute('style', 'display: block');
  }

  stmt.reset();
  stmt.finalize();
  stmt = null;
}

function buildFavoritesPages()
{
  var favDiv = document.getElementById("favorites");
  while (favDiv.childNodes.length > 0)
    favDiv.removeChild(favDiv.childNodes[0]);

  var hs = Components.classes["@mozilla.org/browser/nav-history-service;1"].
    getService(Components.interfaces.nsINavHistoryService);
  var pip = hs.QueryInterface(Components.interfaces.nsPIPlacesDatabase);

  var historyService = Components.classes["@mozilla.org/browser/nav-history-service;1"]
    .getService(Components.interfaces.nsINavHistoryService);
  var db = historyService.QueryInterface(Components.interfaces.nsPIPlacesDatabase).DBConnection;
  // favourite pages by only showing 9 sites from the top 9 domains you visit and then showing
  // the most popular page from each site.
  // the RESULTS_AS_SITE_QUERY + SORT_BY_VISITCOUNT_DESCENDING query only returns sites odered by title
  // so use SQL directly
  var q = "SELECT h.url, h.title FROM " +
            "(SELECT " +
               "(SELECT p.id FROM moz_places p " +
                  "WHERE p.rev_host=s.rev_host ORDER BY p.visit_count DESC" +
               ") as id " +
               "FROM " +
                 "(SELECT rev_host, sum(visit_count) as count " +
                   "FROM moz_places WHERE hidden <> 1 AND " +
                                        "visit_count <> 0 AND " +
                                        "SUBSTR(url, 1, 7) <> 'moblin:' " +
                   "GROUP BY rev_host ORDER BY count DESC LIMIT " + N_FAV_PAGES*4 +
                 ") as s" +
             ") as r " +
          "LEFT JOIN moz_places h ON h.id=r.id"

  var stmt;
  stmt = db.createStatement(q);
  while (favDiv.childNodes.length < N_FAV_PAGES && stmt.executeStep()) {
    var page = {uri: "", title: ""};
    page.uri = stmt.getString(0);
    page.title = stmt.getString(1);

    // Get a local URI for the thumbnail based on the page URL
    var thumb_path = getThumbPath (page.uri);
    var thumb_file = Components.classes["@mozilla.org/file/local;1"]
      .createInstance(Components.interfaces.nsILocalFile);
    thumb_file.initWithPath(thumb_path);
    if (thumb_file.exists() == false) {
      continue;
    }

    var div = document.createElement('div');
    div.setAttribute('class', 'favorite');
    
    // Create elements for the close button. The outer div is
    // just used to make the position for the inner element
    // relative to the top of the favorite
    var closeOuter = document.createElement('div');
    closeOuter.setAttribute('class', 'close-button-outer');
    var closeInner = document.createElement('a');
    closeInner.setAttribute('class', 'close-button-inner');
    closeInner.setAttribute('href', 'javascript:');
    closeOuter.appendChild(closeInner);
    div.appendChild(closeOuter);
    
    // Create a closure with the url and use it as a callback
    // to remove the favorite when the link is clicked
    let (url = page.uri) {
      closeInner.addEventListener("click",
                                  function(e) {
                                    var ioService = Components.classes["@mozilla.org/network/io-service;1"]  
                                      .getService(Components.interfaces.nsIIOService);
                                    var uri = ioService.newURI(url, null, null);
                                    historyService.removePage (uri);
                                    pip.commitPendingChanges();
                                    buildFavoritesPages ();
                                    e.stopPropagation();
                                  },
                                  false);
    }
    
    var a = document.createElement('a');
    div.appendChild(a);
    var thumbnailDiv = document.createElement('div');
    thumbnailDiv.setAttribute('class', 'thumbnail');
    img = document.createElement('img');
    img.setAttribute('class', 'thumbnail');
    thumbnailDiv.appendChild(img);
    a.appendChild(thumbnailDiv);
    var p = document.createElement('p');
    a.appendChild(p);

    p.appendChild(document.createTextNode(page.title));
    a.setAttribute("href", page.uri);
    
    // Get a local URI for the thumbnail based on the page URL
    img.setAttribute("src", "file://" + thumb_path);
    favDiv.appendChild(div);

  }
  if (favDiv.childNodes.length == 0)
    {
      // If there are no pages then display some instructions instead
      var req = new XMLHttpRequest();
      req.open('GET',
               'no-history-fallback.xhtml',
               false);
      req.send(null);
      favDiv.innerHTML = "";
      favDiv.appendChild(req.responseXML.getElementById("no-history-root"));
    }
  stmt.reset();
  stmt.finalize();
  stmt = null;
}

function historyObserver ()
{
  dump ("historyObserver()\n");
}

historyObserver.prototype = 
{
  onBeginUpdateBatch : function () 
  {
  },

  onEndUpdateBatch : function () 
  {
  },

  onClearHistory : function ()
  {
    buildFavoritesPages ();
  },

  onVisit : function (aURI, aVisitID, aTime, aSessionID, aReferringID, aTransitionType, aAdded)
  {
    buildFavoritesPages ();
  },

  onTitleChanged : function (aURI, aPageTitle) {
  },

  onDeleteURI : function (aURI) {
  },

  onPageChanged: function (aURI, aWhat, aValue) {
  },

  onPageExpired: function(aURI, aVisitTime, aWholeEntry) {
  },

  QueryInterface: function(iid) {
    if (iid.equals(Components.interfaces.nsINavHistoryObserver) ||
        iid.equals(Components.interfaces.nsISupports)) {
      return this;
    }
    throw Components.results.NS_ERROR_NO_INTERFACE;
  }
}

function bookmarksObserver ()
{
  dump ("bookmarksObserver()\n");
}

bookmarksObserver.prototype = 
{
  onBeginUpdateBatch : function ()
  {
  },

  onEndUpdateBatch : function ()
  {
  },
  
  onItemAdded : function (aItemId, aParentID, aIndex, aItemType)
  {
    buildPinnedPages ();
  },

  onItemRemoved : function (aItemId, aParentID, aIndex, aItemType)
  {
    buildPinnedPages ();
  },

  onItemChanged : function (aBookmarkId, aProperty, aIsAnnotationProperty, aNewValue, aItemType)
  {
    buildPinnedPages ();
  },

  onItemMoved : function (aItemId, aOldParentId, aOldIndex, aNewParentId, aNewIndex, aItemType)
  {
    buildPinnedPages ();
  },

  onItemVisited : function (aBookmarkId, aVisitID, time)
  {
  },

  QueryInterface: function(iid) {
    if (iid.equals(Components.interfaces.nsINavBookmarkObserver) ||
        iid.equals(Components.interfaces.nsISupports)) {
      return this;
    }
    throw Components.results.NS_ERROR_NO_INTERFACE;
  }
}

function eventListener (event )
{
  buildPinnedPages ();
  buildFavoritesPages ();

  // focus and select the url in url bar
  // make it easy for user to type new url
  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].
    getService(Components.interfaces.nsIWindowMediator);
  var urlbar = wm.getMostRecentWindow("navigator:browser").gURLBar;

  urlbar.focus();
  urlbar.select();

  var historyService = Components.classes["@mozilla.org/browser/nav-history-service;1"]
      .getService(Components.interfaces.nsINavHistoryService);
  var observer = new historyObserver ();
  // Disable the history service observer
  // JS reports "buildFavoritePages is undefined", 
  // fix target for 2.2 release
  //historyService.addObserver (observer, false);
  
  var bookmarkService = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"]
      .getService(Components.interfaces.nsINavBookmarksService);
  var bmObserver = new bookmarksObserver ();
  bookmarkService.addObserver (bmObserver, false);

  // Unregister the load event so we don't initialize more than once
  event.currentTarget.removeEventListener("DOMContentLoaded",
                                          arguments.callee,
                                          false);
}

// Run the initialisation once the page has finished loading
document.addEventListener("DOMContentLoaded", eventListener, false);
